/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: drawingfragment.cxx,v $
 *
 *  $Revision: 1.1.2.2 $
 *
 *  last change: $Author: dr $ $Date: 2008/01/28 09:48:48 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#include "oox/xls/drawingfragment.hxx"
#include "oox/helper/attributelist.hxx"

using ::rtl::OUString;
using ::com::sun::star::uno::Reference;
using ::com::sun::star::xml::sax::XFastContextHandler;

namespace oox {
namespace xls {
namespace drawing {

// ============================================================================

OoxAnchorPosition::OoxAnchorPosition() :
    mnX( 0 ),
    mnY( 0 )
{
}

void OoxAnchorPosition::setOoxValue( sal_Int32 nElement, sal_Int32 nValue )
{
    switch( nElement )
    {
        case XML_x:     mnX = nValue;   break;
        case XML_y:     mnY = nValue;   break;
        default:    OSL_ENSURE( false, "OoxAnchorPosition::setOoxValue - unexpected element" );
    }
}

// ----------------------------------------------------------------------------

OoxAnchorSize::OoxAnchorSize() :
    mnWidth( 0 ),
    mnHeight( 0 )
{
}

void OoxAnchorSize::setOoxValue( sal_Int32 nElement, sal_Int32 nValue )
{
    switch( nElement )
    {
        case XML_cx:    mnWidth = nValue;   break;
        case XML_cy:    mnHeight = nValue;  break;
        default:    OSL_ENSURE( false, "OoxAnchorSize::setOoxValue - unexpected element" );
    }
}

// ----------------------------------------------------------------------------

OoxCellAnchorPoint::OoxCellAnchorPoint() :
    mnCol( 0 ),
    mnColOffset( 0 ),
    mnRow( 0 ),
    mnRowOffset( 0 )
{
}

void OoxCellAnchorPoint::setOoxValue( sal_Int32 nElement, sal_Int32 nValue )
{
    switch( nElement )
    {
        case XDR_TOKEN( col ):      mnCol = nValue;         break;
        case XDR_TOKEN( colOff ):   mnColOffset = nValue;   break;
        case XDR_TOKEN( row ):      mnRow = nValue;         break;
        case XDR_TOKEN( rowOff ):   mnRowOffset = nValue;   break;
        default:    OSL_ENSURE( false, "OoxCellAnchorPoint::setOoxValue - unexpected element" );
    }
}

// ----------------------------------------------------------------------------

OoxAnchorClientData::OoxAnchorClientData() :
    mbLocksWithSheet( true ),
    mbPrintsWithSheet( true )
{
}

// ============================================================================

AnchorBase::AnchorBase( const WorksheetHelper& rHelper ) :
    WorksheetHelper( rHelper )
{
}

void AnchorBase::importPos( const AttributeList& rAttribs )
{
    setValue( XDR_TOKEN( pos ), XML_x, rAttribs.getInteger( XML_x, 0 ) );
    setValue( XDR_TOKEN( pos ), XML_y, rAttribs.getInteger( XML_y, 0 ) );
}

void AnchorBase::importExt( const AttributeList& rAttribs )
{
    setValue( XDR_TOKEN( ext ), XML_cx, rAttribs.getInteger( XML_cx, 0 ) );
    setValue( XDR_TOKEN( ext ), XML_cy, rAttribs.getInteger( XML_cy, 0 ) );
}

void AnchorBase::importClientData( const AttributeList& rAttribs )
{
    maClientData.mbLocksWithSheet  = rAttribs.getBool( XML_fLocksWithSheet, true );
    maClientData.mbPrintsWithSheet = rAttribs.getBool( XML_fPrintsWithSheet, true );
}

void AnchorBase::importAnchor( const AttributeList& )
{
}

// ----------------------------------------------------------------------------

AbsoluteAnchor::AbsoluteAnchor( const WorksheetHelper& rHelper ) :
    AnchorBase( rHelper )
{
}

void AbsoluteAnchor::setValue( sal_Int32 nParentElement, sal_Int32 nElement, sal_Int32 nValue )
{
    switch( nParentElement )
    {
        case XDR_TOKEN( pos ):  maPos.setOoxValue( nElement, nValue );  break;
        case XDR_TOKEN( ext ):  maSize.setOoxValue( nElement, nValue ); break;
        default:                OSL_ENSURE( false, "AbsoluteAnchor::setValue - unexpected parent element" );
    }
}

// ----------------------------------------------------------------------------

OneCellAnchor::OneCellAnchor( const WorksheetHelper& rHelper ) :
    AnchorBase( rHelper )
{
}

void OneCellAnchor::setValue( sal_Int32 nParentElement, sal_Int32 nElement, sal_Int32 nValue )
{
    switch( nParentElement )
    {
        case XDR_TOKEN( from ): maFrom.setOoxValue( nElement, nValue ); break;
        case XDR_TOKEN( ext ):  maSize.setOoxValue( nElement, nValue ); break;
        default:                OSL_ENSURE( false, "OneCellAnchor::setValue - unexpected parent element" );
    }
}

// ----------------------------------------------------------------------------

TwoCellAnchor::TwoCellAnchor( const WorksheetHelper& rHelper ) :
    AnchorBase( rHelper ),
    mnEditAs( XML_twoCell )
{
}

void TwoCellAnchor::setValue( sal_Int32 nParentElement, sal_Int32 nElement, sal_Int32 nValue )
{
    switch( nParentElement )
    {
        case XDR_TOKEN( from ): maFrom.setOoxValue( nElement, nValue ); break;
        case XDR_TOKEN( to ):   maTo.setOoxValue( nElement, nValue );   break;
        default:                OSL_ENSURE( false, "TwoCellAnchor::setValue - unexpected parent element" );
    }
}

void TwoCellAnchor::importAnchor( const AttributeList& rAttribs )
{
    mnEditAs = rAttribs.getToken( XML_editAs, XML_twoCell );
}

// ============================================================================

OoxDrawingFragment::OoxDrawingFragment( const WorksheetHelper& rHelper, const OUString& rFragmentPath ) :
    OoxWorksheetFragmentBase( rHelper, rFragmentPath )
{
}

// oox.xls.OoxContextHelper interface -----------------------------------------

OoxContextWrapper OoxDrawingFragment::onCreateContext( sal_Int32 nElement, const AttributeList& )
{
    switch( getCurrentContext() )
    {
        case XML_ROOT_CONTEXT:
            return  (nElement == XDR_TOKEN( wsDr ));
        case XDR_TOKEN( wsDr ):
            return  (nElement == XDR_TOKEN( absoluteAnchor )) ||
                    (nElement == XDR_TOKEN( oneCellAnchor )) ||
                    (nElement == XDR_TOKEN( twoCellAnchor ));
        case XDR_TOKEN( absoluteAnchor ):
            return  (nElement == XDR_TOKEN( pos )) ||
                    (nElement == XDR_TOKEN( ext )) ||
                    (nElement == XDR_TOKEN( clientData ));
        case XDR_TOKEN( oneCellAnchor ):
            return  (nElement == XDR_TOKEN( from )) ||
                    (nElement == XDR_TOKEN( ext )) ||
                    (nElement == XDR_TOKEN( clientData ));
        case XDR_TOKEN( twoCellAnchor ):
            return  (nElement == XDR_TOKEN( from )) ||
                    (nElement == XDR_TOKEN( to )) ||
                    (nElement == XDR_TOKEN( clientData ));
        case XDR_TOKEN( from ):
        case XDR_TOKEN( to ):
            return  (nElement == XDR_TOKEN( col )) ||
                    (nElement == XDR_TOKEN( colOff )) ||
                    (nElement == XDR_TOKEN( row )) ||
                    (nElement == XDR_TOKEN( rowOff ));
    }
    return false;
}

void OoxDrawingFragment::onStartElement( const AttributeList& rAttribs )
{
    switch( getCurrentContext() )
    {
        case XDR_TOKEN( absoluteAnchor ):
            mxAnchor.reset( new AbsoluteAnchor( *this ) );
            mxAnchor->importAnchor( rAttribs );
        break;
        case XDR_TOKEN( oneCellAnchor ):
            mxAnchor.reset( new OneCellAnchor( *this ) );
            mxAnchor->importAnchor( rAttribs );
        break;
        case XDR_TOKEN( twoCellAnchor ):
            mxAnchor.reset( new TwoCellAnchor( *this ) );
            mxAnchor->importAnchor( rAttribs );
        break;
        case XDR_TOKEN( pos ):
            if( mxAnchor.get() ) mxAnchor->importPos( rAttribs );
        break;
        case XDR_TOKEN( ext ):
            if( mxAnchor.get() ) mxAnchor->importExt( rAttribs );
        break;
        case XDR_TOKEN( clientData ):
            if( mxAnchor.get() ) mxAnchor->importClientData( rAttribs );
        break;
    }
}

void OoxDrawingFragment::onEndElement( const OUString& rChars )
{
    switch( getCurrentContext() )
    {
        case XDR_TOKEN( col ):
        case XDR_TOKEN( colOff ):
        case XDR_TOKEN( row ):
        case XDR_TOKEN( rowOff ):
            if( mxAnchor.get() ) mxAnchor->setValue( getPreviousContext(), getCurrentContext(), rChars.toInt32() );
        break;
        case XDR_TOKEN( absoluteAnchor ):
        case XDR_TOKEN( oneCellAnchor ):
        case XDR_TOKEN( twoCellAnchor ):
            mxAnchor.reset();
        break;
    }
}

// oox.xls.OoxFragmentHandler interface ---------------------------------------

void OoxDrawingFragment::initializeImport()
{
}

void OoxDrawingFragment::finalizeImport()
{
}

// ============================================================================

} // namespace drawing
} // namespace xls
} // namespace oox

